import { Collection, Entity, ManyToOne, OneToMany, PrimaryKey, Property } from '@mikro-orm/core';
import { Sprite } from './sprite';
import { CharacterItem } from './characterItem';
import { ItemType, ItemRarity } from '../utilities/enums';

@Entity()
export class Item {
  @PrimaryKey()
  id!: string;

  @Property()
  name!: string;

  @Property({ nullable: true })
  description?: string;

  @Property()
  itemType!: ItemType;

  @Property()
  stackable = false;

  @Property()
  rarity: ItemRarity = ItemRarity.COMMON;

  @ManyToOne(() => Sprite, { nullable: true })
  sprite?: Sprite;

  @Property()
  createdAt = new Date();

  @Property()
  updatedAt = new Date();

  @OneToMany(() => CharacterItem, characterItem => characterItem.item)
  characters = new Collection<CharacterItem>(this);
}